"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mermaidTemplate = void 0;
const typescript_1 = require("typescript");
const tsuml2_settings_1 = require("../tsuml2-settings");
exports.mermaidTemplate = {
    composition: "+->",
    implements: (interf, implementation) => {
        return (`${exports.mermaidTemplate.plainClassOrInterface(interf)}<|..${exports.mermaidTemplate.plainClassOrInterface(implementation)}`);
    },
    extends: (base, derived) => {
        return `${exports.mermaidTemplate.plainClassOrInterface(base)}<|--${exports.mermaidTemplate.plainClassOrInterface(derived)}`;
    },
    plainClassOrInterface: (name) => escapeMermaid(name),
    class: (name, props, methods) => {
        return `class ${escapeMermaid(name)}{
            ${props.map(propertyTemplate).join("\n")}
            ${methods.map(methodTemplate).join("\n")}
        }`;
    },
    interface: (name, props, methods) => {
        return `class ${escapeMermaid(name)} {
            <<interface>>
            ${props.map(propertyTemplate).join("\n")}
            ${methods.map(methodTemplate).join("\n")}
        }`;
    },
    type: (name, props, methods) => {
        return `class ${escapeMermaid(name)} {
            <<type>>
            ${props.map(propertyTemplate).join("\n")}
            ${methods.map(methodTemplate).join("\n")}
        }`;
    },
    enum: (name, enumItems) => {
        return `class ${escapeMermaid(name)} {
        <<enumeration>>
        ${enumItems.join("\n")}
      }`;
    },
    memberAssociation: memberAssociation
};
function propertyTemplate(property) {
    let retVal = property.name;
    if (property.type && tsuml2_settings_1.SETTINGS.propertyTypes) {
        if (property?.optional) {
            retVal += "?";
        }
        retVal += ": " + escapeMermaid(property.type);
    }
    return applyModifiers(property.modifierFlags, retVal);
}
function methodTemplate(method) {
    let retVal = method.name + "()";
    if (method.returnType && tsuml2_settings_1.SETTINGS.propertyTypes) {
        retVal += " " + escapeMermaid(method.returnType);
    }
    return applyModifiers(method.modifierFlags, retVal);
}
function applyModifiers(modifierFlags, method) {
    if (!tsuml2_settings_1.SETTINGS.modifiers) {
        return method;
    }
    let retVal = "";
    if (modifierFlags & typescript_1.ModifierFlags.Private) {
        retVal = "-";
    }
    else if (modifierFlags & typescript_1.ModifierFlags.Protected) {
        retVal = "#";
    }
    else {
        retVal = "+";
    }
    retVal += method;
    // UML2: static member will be underlined 
    if (modifierFlags & typescript_1.ModifierFlags.Static) {
        retVal = retVal + "$";
    }
    // abstract member will be italic
    if (modifierFlags & typescript_1.ModifierFlags.Abstract) {
        retVal = retVal + "*";
    }
    return retVal;
}
function memberAssociation(association) {
    const multiplicityA = association.a.multiplicity ? `"${association.a.multiplicity}"` : "";
    const multiplicityB = association.b.multiplicity ? `"${association.b.multiplicity}"` : "";
    return `${exports.mermaidTemplate.plainClassOrInterface(association.a.name)} ${multiplicityA} -- ${multiplicityB} ${exports.mermaidTemplate.plainClassOrInterface(association.b.name)}`;
}
// utility functions
function escapeMermaid(str) {
    return str.replace(/[<>]/g, '~');
}
//# sourceMappingURL=mermaid-template.js.map